from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac import Client


class TestSystemBackendMethods(TestCase):
    """Unit tests providing coverage for Vault system backend-related methods in the hvac Client class."""

    @parameterized.expand(
        [
            ("pki lease ID", "pki/issue/my-role/12c7e036-b59e-5e79-3370-03826fc6f34b"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_lease(self, test_label, test_lease_id, requests_mocker):
        test_path = "http://localhost:8200/v1/sys/leases/lookup"
        mock_response = {
            "issue_time": "2018-07-15T08:35:34.775859245-05:00",
            "renewable": False,
            "id": test_lease_id,
            "ttl": 259199,
            "expire_time": "2018-07-18T08:35:34.00004241-05:00",
            "last_renewal": None,
        }
        requests_mocker.register_uri(
            method="PUT",
            url=test_path,
            json=mock_response,
        )
        client = Client()
        response = client.sys.read_lease(
            lease_id=test_lease_id,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )



from hvac.api.auth_methods import JWT

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter

import json


class ValueChecker:
    def __init__(self, expected_body_params, return_status_code):
        self.expected_body_params = expected_body_params
        self.return_status_code = return_status_code
        self.expected_headers = {
            "X-Vault-Request": "true",
            "Content-Type": "application/json",
        }

    def text_callback(self, request, context):
        rq_params = json.loads(request.body)
        assert rq_params == self.expected_body_params
        rq_headers = {
            k: v for k, v in request.headers.items() if k in self.expected_headers
        }
        assert rq_headers == self.expected_headers
        context.status_code = self.return_status_code


@parameterized.expand(
    [
        (
            "hvac",
            "https://vault/user",
            ["https://localhost:8200/jwt-test/callback"],
            "jwt",
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            "string",
            False,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
        ),
        (
            "hvac",
            "https://vault/user",
            ["https://localhost:8200/jwt-test/callback"],
            "jwt",
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            "string",
            False,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            False,
        ),
        (
            "hvac",
            "https://vault/user",
            ["https://localhost:8200/jwt-test/callback"],
            "jwt",
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            "string",
            False,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            True,
        ),
    ]
)
@requests_mock.Mocker()
def test_create_role(
    name,
    user_claim,
    allowed_redirect_uris,
    role_type,  # = "jwt"
    bound_audiences,
    clock_skew_leeway,
    expiration_leeway,
    not_before_leeway,
    bound_subject,
    bound_claims,
    groups_claim,
    claim_mappings,
    oidc_scopes,
    bound_claims_type,  # ="string"
    verbose_oidc_logging,  # =False,
    token_ttl,
    token_max_ttl,
    token_policies,
    token_bound_cidrs,
    token_explicit_max_ttl,
    token_no_default_policy,
    token_num_uses,
    token_period,
    token_type,
    path,
    user_claim_json_pointer,
    requests_mocker,
):

    test_arguments = {
        "name": name,
        "role_type": role_type,
        "bound_audiences": bound_audiences,
        "user_claim": user_claim,
        "clock_skew_leeway": clock_skew_leeway,
        "expiration_leeway": expiration_leeway,
        "not_before_leeway": not_before_leeway,
        "bound_subject": bound_subject,
        "bound_claims": bound_claims,
        "groups_claim": groups_claim,
        "claim_mappings": claim_mappings,
        "oidc_scopes": oidc_scopes,
        "allowed_redirect_uris": allowed_redirect_uris,
        "bound_claims_type": bound_claims_type,
        "verbose_oidc_logging": verbose_oidc_logging,
        "token_ttl": token_ttl,
        "token_max_ttl": token_max_ttl,
        "token_policies": token_policies,
        "token_bound_cidrs": token_bound_cidrs,
        "token_explicit_max_ttl": token_explicit_max_ttl,
        "token_no_default_policy": token_no_default_policy,
        "token_num_uses": token_num_uses,
        "token_period": token_period,
        "token_type": token_type,
        "user_claim_json_pointer": user_claim_json_pointer,
    }

    check_arguments = {k: v for k, v in test_arguments.items() if v is not None}
    expected_status_code = 204

    eff_path = "jwt" if path is None else path
    mock_url = f"http://localhost:8200/v1/auth/{eff_path}/role/{name}"
    requests_mocker.register_uri(
        method="POST",
        url=mock_url,
        text=ValueChecker(check_arguments, expected_status_code).text_callback,
    )
    jwt = JWT(adapter=JSONAdapter())
    actual_response = jwt.create_role(**test_arguments)
    assert expected_status_code == actual_response.status_code



import logging
import time

import jwcrypto.jwk as jwk
from authlib.integrations.flask_oauth2 import current_token
from authlib.oauth2 import OAuth2Error
from flask import Blueprint, jsonify, request, session, url_for
from werkzeug.security import gen_salt

from tests.utils import get_config_file_path
from tests.utils.mock_oauth_provider.models import OAuth2Client, User, db
from tests.utils.mock_oauth_provider.oauth2 import authorization, require_oauth

logger = logging.getLogger(__name__)
bp = Blueprint("mock-oauth-provider", "home")


def current_user():
    if "id" in session:
        uid = session["id"]
        return User.query.get(uid)
    return None


@bp.route("/api/user", methods=("GET", "POST"))
def user():
    user = current_user()

    if request.method == "POST":
        username = request.form.get("username")
        user = User.query.filter_by(username=username).first()
        if not user:
            user = User(username=username)
            db.session.add(user)
            db.session.commit()
        session["id"] = user.id

    clients = []
    if user:
        clients = OAuth2Client.query.filter_by(user=user).all()

    response = {
        "user": user.as_dict() if user else None,
        "clients": {c.client_metadata["client_name"]: c.as_dict() for c in clients},
    }
    return jsonify(response)


@bp.route("/api/create_client", methods=["POST"])
def create_client():
    user = current_user()
    if not user:
        response = jsonify(
            {
                "message": "Unauthorized, no active user session found",
            }
        )
        return response, 401

    form = request.form
    client_id = gen_salt(24)
    client = OAuth2Client(client_id=client_id, user_id=user.id)
    # Mixin doesn't set the issue_at date
    client.client_id_issued_at = int(time.time())
    if client.token_endpoint_auth_method == "none":
        client.client_secret = ""
    else:
        client.client_secret = gen_salt(48)

    client_metadata = {
        "client_name": form["client_name"],
        "client_uri": form["client_uri"],
        "grant_types": form["grant_types"],
        "redirect_uris": form["redirect_uris"],
        "response_types": form["response_types"],
        "scope": form["scope"],
        "token_endpoint_auth_method": form["token_endpoint_auth_method"],
    }
    client.set_client_metadata(client_metadata)
    db.session.add(client)
    db.session.commit()
    return jsonify(client.as_dict())


@bp.route("/oauth/.well-known/openid-configuration", methods=["GET"])
def well_known():
    base_url = request.host_url.rstrip("/")
    response = {
        "issuer": f"{base_url}/oauth",
        "authorization_endpoint": "{url}{authorize}".format(
            url=base_url, authorize=url_for(".authorize")
        ),
        "token_endpoint": "{url}{issue}".format(
            url=base_url, issue=url_for(".issue_token")
        ),
        "jwks_uri": "{url}{get_keys}".format(
            url=base_url, get_keys=url_for(".get_keys")
        ),
    }
    return jsonify(response)


@bp.route("/oauth/authorize", methods=["GET", "POST"])
def authorize():
    user = current_user()
    if request.method == "GET":
        try:
            logger.debug("get_consent_grant for user: %s" % user)
            grant = authorization.get_consent_grant(end_user=user)
            logger.debug(f"grant for user {user}: {grant}")
        except OAuth2Error as error:
            return jsonify(dict(error.get_body()))
        return jsonify(
            {
                "requesting_client": grant.client.client_name,
                "requested_scope": grant.request.scope,
                "user": user.username,
            }
        )
    if not user and "username" in request.form:
        username = request.form.get("username")
        user = User.query.filter_by(username=username).first()
    if request.form["confirm"]:
        grant_user = user
    else:
        grant_user = None
    return authorization.create_authorization_response(grant_user=grant_user)


@bp.route("/oauth/token", methods=["POST"])
def issue_token():
    return authorization.create_token_response()


@bp.route("/oauth/keys")
def get_keys():
    with open(get_config_file_path("oidc_private.pem"), "rb") as fh:
        signing_key = jwk.JWK.from_pem(fh.read())._public_params()
    return jsonify(
        {
            "keys": [
                signing_key,
            ]
        }
    )


@bp.route("/api/me")
@require_oauth("profile")
def api_me():
    user = current_token.user
    return jsonify(id=user.id, username=user.username)



import logging
from unittest import TestCase
from unittest import skipIf

from parameterized import parameterized

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(
    utils.vault_version_lt("0.11.0"),
    "Azure secret engine not available before Vault version 0.11.0",
)
class TestAzure(HvacIntegrationTestCase, TestCase):
    TENANT_ID = "00000000-0000-0000-0000-000000000000"
    SUBSCRIPTION_ID = "00000000-0000-0000-0000-000000000000"
    DEFAULT_MOUNT_POINT = "azure-integration-test"

    def setUp(self):
        super().setUp()
        self.client.sys.enable_secrets_engine(
            backend_type="azure",
            path=self.DEFAULT_MOUNT_POINT,
        )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(path=self.DEFAULT_MOUNT_POINT)
        super().tearDown()

    @parameterized.expand(
        [
            ("no parameters",),
            ("valid environment argument", "AzureUSGovernmentCloud"),
            (
                "invalid environment argument",
                "AzureCityKity",
                exceptions.ParamValidationError,
                "invalid environment argument provided",
            ),
        ]
    )
    def test_configure_and_read_configuration(
        self, test_label, environment=None, raises=False, exception_message=""
    ):
        configure_arguments = {
            "subscription_id": self.SUBSCRIPTION_ID,
            "tenant_id": self.TENANT_ID,
            "mount_point": self.DEFAULT_MOUNT_POINT,
        }
        if environment is not None:
            configure_arguments["environment"] = environment
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.azure.configure(**configure_arguments)
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            configure_response = self.client.secrets.azure.configure(
                **configure_arguments
            )
            logging.debug("configure_response: %s" % configure_response)
            read_configuration_response = self.client.secrets.azure.read_config(
                mount_point=self.DEFAULT_MOUNT_POINT,
            )
            logging.debug(
                "read_configuration_response: %s" % read_configuration_response
            )
            # raise Exception()
            self.assertEqual(
                first=self.SUBSCRIPTION_ID,
                second=read_configuration_response["subscription_id"],
            )
            self.assertEqual(
                first=self.TENANT_ID,
                second=read_configuration_response["tenant_id"],
            )
            if environment is not None:
                self.assertEqual(
                    first=environment,
                    second=read_configuration_response["environment"],
                )

    @parameterized.expand(
        [
            ("create and then delete config",),
        ]
    )
    def test_delete_config(self, test_label):
        configure_response = self.client.secrets.azure.configure(
            subscription_id=self.SUBSCRIPTION_ID,
            tenant_id=self.TENANT_ID,
            mount_point=self.DEFAULT_MOUNT_POINT,
        )
        logging.debug("configure_response: %s" % configure_response)
        self.client.secrets.azure.delete_config(
            mount_point=self.DEFAULT_MOUNT_POINT,
        )
        read_configuration_response = self.client.secrets.azure.read_config(
            mount_point=self.DEFAULT_MOUNT_POINT,
        )
        logging.debug("read_configuration_response: %s" % read_configuration_response)
        read_expected_response = {
            "client_id": "",
            "environment": "",
            "subscription_id": "",
            "tenant_id": "",
        }
        if utils.vault_version_ge("1.9.0"):
            read_expected_response["root_password_ttl"] = 0
            if utils.vault_version_lt("1.12.0"):
                read_expected_response["use_microsoft_graph_api"] = False
        self.assertEqual(
            first=read_expected_response,
            second=read_configuration_response,
        )



from textwrap import dedent
from unittest import TestCase

import requests_mock
from parameterized import parameterized, param

from hvac.adapters import JSONAdapter
from hvac.api.secrets_engines import Gcp
from hvac.exceptions import (
    ParamValidationError,
    InvalidRequest,
    InvalidPath,
    UnexpectedError,
    InternalServerError,
)

TEST_MOUNT_POINT = "gcp-test"
TEST_ROLESET_NAME = "hvac-roleset"
TEST_SERVICE_ACCOUNT_ID = "test-hvac-sa"
TEST_PROJECT_ID = "test-hvac"
TEST_STATIC_ACCOUNT_NAME = "hvac-static-account"
TEST_IMPERSONATED_ACCOUNT_NAME = "hvac-impersonated-account"
TEST_SERVICE_ACCOUNT_EMAIL = (
    f"{TEST_SERVICE_ACCOUNT_ID}@{TEST_PROJECT_ID}.iam.gserviceaccount.com"
)
DEFAULT_CREDENTIALS = dedent(
    """
    {
        "project_id": "test-hvac-project-not-a-real-project",
        "private_key_id": "3900c1e6f6720d770a6af25710cad56696272f8d",
    }
    """
)
DEFAULT_BINDINGS = dedent(
    """
    resource "//cloudresourcemanager.googleapis.com/projects/mygcpproject" {
      roles = [
        "roles/viewer"
      ],
    }
    """
)
DEFAULT_TOKEN_SCOPES = ["https://www.googleapis.com/auth/cloud-platform"]


class TestGcp(TestCase):
    def setUp(self):
        self._json_adapter = Gcp(adapter=JSONAdapter())

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204),
            param(
                method="POST",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={"errors": ["error parsing JSON"]},
            ),
        ]
    )
    def test_configure(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/config".format(
            mount_point=TEST_MOUNT_POINT,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                invalid_json = {"project_id": None}

                with self.assertRaises(raises) as cm:

                    self._json_adapter.configure(
                        credentials=invalid_json,
                        ttl=3600,
                        max_ttl=14400,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.configure(
                    credentials=DEFAULT_CREDENTIALS,
                    ttl=3600,
                    max_ttl=14400,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)

    @parameterized.expand(
        [
            param(
                method="POST",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "private_key_id": "bb9b47e33693847c8281a959c4286eb848be909c"
                    }
                },
            ),
            param(
                method="POST",
                expected_status_code=404,
                raises=InvalidPath,
                expected_response={"errors": ["no handler for route"]},
            ),
            param(
                method="POST",
                expected_status_code=500,
                raises=InternalServerError,
                expected_response={
                    "errors": ["1 error occurred: failed to create new key"]
                },
            ),
        ]
    )
    def test_rotate_root_credentials(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/config/rotate-root".format(
            mount_point=TEST_MOUNT_POINT,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.rotate_root_credentials(
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.rotate_root_credentials(
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={"data": {"max_ttl": 0, "ttl": 0}},
            ),
            param(
                method="GET",
                expected_status_code=404,
                raises=InvalidPath,
                expected_response={"errors": ["no handler for route"]},
            ),
        ]
    )
    def test_read_config(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/config".format(
            mount_point=TEST_MOUNT_POINT,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.read_config(
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(expected_response, cm.exception.json)
            else:
                resp = self._json_adapter.read_config(
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204, secret_type="access_token"),
            param(
                method="POST",
                expected_status_code=204,
                secret_type="service_account_key",
            ),
            param(
                method="POST",
                expected_status_code=204,
                secret_type="invalid_secret_type",
                raises=ParamValidationError,
            ),
        ]
    )
    def test_create_or_update_roleset(
        self,
        method,
        expected_status_code,
        secret_type,
        raises=None,
        expected_response=None,
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/roleset/{name}".format(
            mount_point=TEST_MOUNT_POINT,
            name=TEST_ROLESET_NAME,
        )

        token_scopes = DEFAULT_TOKEN_SCOPES if secret_type == "access_token" else None

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.create_or_update_roleset(
                        name=TEST_ROLESET_NAME,
                        project=TEST_PROJECT_ID,
                        bindings=DEFAULT_BINDINGS,
                        secret_type=secret_type,
                        token_scopes=token_scopes,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(
                    member="unsupported secret_type argument provided",
                    container=str(cm.exception),
                )
            else:
                resp = self._json_adapter.create_or_update_roleset(
                    name=TEST_ROLESET_NAME,
                    project=TEST_PROJECT_ID,
                    bindings=DEFAULT_BINDINGS,
                    secret_type=secret_type,
                    token_scopes=token_scopes,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204),
            param(method="POST", expected_status_code=400, raises=InvalidRequest),
            param(method="POST", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_rotate_roleset_account(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/roleset/{name}/rotate".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_ROLESET_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.rotate_roleset_account(
                        name=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.rotate_roleset_account(
                    name=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204),
            param(method="POST", expected_status_code=400, raises=InvalidRequest),
            param(method="POST", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_rotate_roleset_account_key(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/roleset/{name}/rotate-key".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_ROLESET_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.rotate_roleset_account_key(
                        name=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.rotate_roleset_account_key(
                    name=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                secret_type="access_token",
                expected_response={
                    "data": {
                        "secret_type": "access_token",
                        "service_account_email": TEST_SERVICE_ACCOUNT_EMAIL,
                        "service_account_project": TEST_PROJECT_ID,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=200,
                secret_type="service_account_key",
                expected_response={
                    "data": {
                        "secret_type": "service_account_key",
                        "service_account_email": TEST_SERVICE_ACCOUNT_EMAIL,
                        "service_account_project": TEST_PROJECT_ID,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=404,
                secret_type="access_token",
                raises=InvalidPath,
            ),
            param(
                method="GET",
                expected_status_code=405,
                secret_type="service_account_key",
                raises=UnexpectedError,
            ),
        ]
    )
    def test_read_roleset(
        self,
        method,
        expected_status_code,
        secret_type,
        raises=None,
        expected_response=None,
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/roleset/{name}".format(
            mount_point=TEST_MOUNT_POINT,
            name=TEST_ROLESET_NAME,
        )

        if expected_response is not None and secret_type == "access_token":
            expected_response["data"]["token_scopes"] = DEFAULT_TOKEN_SCOPES

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.read_roleset(
                        name=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.read_roleset(
                    name=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="LIST",
                expected_status_code=200,
                expected_response={"data": {"keys": ["roleset-01", "roleset-02"]}},
            ),
            param(method="LIST", expected_status_code=404, raises=InvalidPath),
            param(method="LIST", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_list_rolesets(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/rolesets".format(
            mount_point=TEST_MOUNT_POINT,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.list_rolesets(
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.list_rolesets(
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="DELETE", expected_status_code=204),
            param(method="DELETE", expected_status_code=404, raises=InvalidPath),
            param(method="DELETE", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_delete_roleset(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/roleset/{name}".format(
            mount_point=TEST_MOUNT_POINT,
            name=TEST_ROLESET_NAME,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.delete_roleset(
                        name=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.delete_roleset(
                    name=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "expires_at_seconds": 1679109162,
                        "token": "ya29.c.b0Aaekm1Le-n2NCqrzZjdMtjpbgRji2yhiJkO...",
                        "token_ttl": 3598,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={
                    "errors": ["role set 'missing-roleset' does not exists"]
                },
            ),
        ]
    )
    def test_generate_oauth2_access_token(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/token/{roleset}".format(
            mount_point=TEST_MOUNT_POINT,
            roleset=TEST_ROLESET_NAME,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.generate_oauth2_access_token(
                        roleset=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.generate_oauth2_access_token(
                    roleset=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "key_algorithm": "KEY_ALG_RSA_2048",
                        "key_type": "TYPE_GOOGLE_CREDENTIALS_FILE",
                        "private_key_data": "ewogICJ0eXBlIjogInNlcnZpY2VfYWNjb3VudCIsCiAgIn...",
                    }
                },
            ),
            param(
                method="POST",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "key_algorithm": "KEY_ALG_RSA_1024",
                        "key_type": "TYPE_GOOGLE_CREDENTIALS_FILE",
                        "private_key_data": "ewogICJ0eXBlIjogInNlcnZpY2VfYWNjb3VudCIsCiAgIn...",
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={
                    "errors": ["role set 'missing-roleset' does not exists"]
                },
            ),
        ]
    )
    def test_generate_service_account_key(
        self,
        method,
        expected_status_code,
        raises=None,
        expected_response=None,
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/key/{roleset}".format(
            mount_point=TEST_MOUNT_POINT,
            roleset=TEST_ROLESET_NAME,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.generate_service_account_key(
                        roleset=TEST_ROLESET_NAME,
                        mount_point=TEST_MOUNT_POINT,
                        method=method,
                    )

                self.assertIn(member=mock_url, container=str(cm.exception))
            else:
                resp = self._json_adapter.generate_service_account_key(
                    roleset=TEST_ROLESET_NAME,
                    mount_point=TEST_MOUNT_POINT,
                    method=method,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204, secret_type="access_token"),
            param(
                method="POST",
                expected_status_code=204,
                secret_type="service_account_key",
            ),
            param(
                method="POST",
                expected_status_code=204,
                secret_type="invalid_secret_type",
                raises=ParamValidationError,
            ),
        ]
    )
    def test_create_or_update_static_account(
        self,
        method,
        expected_status_code,
        secret_type,
        raises=None,
        expected_response=None,
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/static-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_STATIC_ACCOUNT_NAME,
            )
        )

        token_scopes = DEFAULT_TOKEN_SCOPES if secret_type == "access_token" else None

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.create_or_update_static_account(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        service_account_email=TEST_SERVICE_ACCOUNT_EMAIL,
                        bindings=DEFAULT_BINDINGS,
                        secret_type=secret_type,
                        token_scopes=token_scopes,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(
                    member="unsupported secret_type argument provided",
                    container=str(cm.exception),
                )
            else:
                resp = self._json_adapter.create_or_update_static_account(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    service_account_email=TEST_SERVICE_ACCOUNT_EMAIL,
                    bindings=DEFAULT_BINDINGS,
                    secret_type=secret_type,
                    token_scopes=token_scopes,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204),
            param(method="POST", expected_status_code=400, raises=InvalidRequest),
            param(
                method="POST",
                expected_status_code=405,
                raises=UnexpectedError,
                expected_response={
                    "errors": ["cannot rotate key for non-access-token static account"]
                },
            ),
        ]
    )
    def test_rotate_static_account_key(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/static-account/{name}/rotate-key".format(
            mount_point=TEST_MOUNT_POINT,
            name=TEST_STATIC_ACCOUNT_NAME,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.rotate_static_account_key(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.rotate_static_account_key(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                secret_type="access_token",
                expected_response={
                    "data": {
                        "secret_type": "access_token",
                        "service_account_email": TEST_SERVICE_ACCOUNT_EMAIL,
                        "service_account_project": TEST_PROJECT_ID,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=200,
                secret_type="service_account_key",
                expected_response={
                    "data": {
                        "secret_type": "service_account_key",
                        "service_account_email": TEST_SERVICE_ACCOUNT_EMAIL,
                        "service_account_project": TEST_PROJECT_ID,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=404,
                secret_type="access_token",
                raises=InvalidPath,
            ),
            param(
                method="GET",
                expected_status_code=405,
                secret_type="service_account_key",
                raises=UnexpectedError,
            ),
        ]
    )
    def test_read_static_account(
        self,
        method,
        expected_status_code,
        secret_type,
        raises=None,
        expected_response=None,
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/static-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_STATIC_ACCOUNT_NAME,
            )
        )

        if expected_response is not None and secret_type == "access_token":
            expected_response["data"]["token_scopes"] = DEFAULT_TOKEN_SCOPES

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.read_static_account(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.read_static_account(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="LIST",
                expected_status_code=200,
                expected_response={
                    "data": {"keys": ["static-account-01", "static-account-02"]}
                },
            ),
            param(method="LIST", expected_status_code=404, raises=InvalidPath),
            param(method="LIST", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_list_static_accounts(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/static-accounts".format(
            mount_point=TEST_MOUNT_POINT,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.list_static_accounts(
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.list_static_accounts(
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="DELETE", expected_status_code=204),
            param(method="DELETE", expected_status_code=404, raises=InvalidPath),
            param(method="DELETE", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_delete_static_account(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/static-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_STATIC_ACCOUNT_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.delete_static_account(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.delete_static_account(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "expires_at_seconds": 1679109162,
                        "token": "ya29.c.b0Aaekm1Le-n2NCqrzZjdMtjpbgRji2yhiJkO...",
                        "token_ttl": 3598,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={
                    "errors": ['static account "missing-account" does not exists'],
                },
            ),
        ]
    )
    def test_generate_static_account_oauth2_access_token(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/static-account/{name}/token".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_STATIC_ACCOUNT_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.generate_static_account_oauth2_access_token(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.generate_static_account_oauth2_access_token(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "key_algorithm": "KEY_ALG_RSA_2048",
                        "key_type": "TYPE_GOOGLE_CREDENTIALS_FILE",
                        "private_key_data": "ewogICJ0eXBlIjogInNlcnZpY2VfYWNjb3VudCIsCiAgIn...",
                    }
                },
            ),
            param(
                method="POST",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "key_algorithm": "KEY_ALG_RSA_1024",
                        "key_type": "TYPE_GOOGLE_CREDENTIALS_FILE",
                        "private_key_data": "ewogICJ0eXBlIjogInNlcnZpY2VfYWNjb3VudCIsCiAgIn...",
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={
                    "errors": ['static account "missing-account" does not exists'],
                },
            ),
        ]
    )
    def test_generate_static_account_service_account_key(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/static-account/{name}/key".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_STATIC_ACCOUNT_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.generate_static_account_service_account_key(
                        name=TEST_STATIC_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                        method=method,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.generate_static_account_service_account_key(
                    name=TEST_STATIC_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                    method=method,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="POST", expected_status_code=204, ttl="3h"),
            param(
                method="POST",
                expected_status_code=400,
                ttl="invalid",
                raises=InvalidRequest,
                expected_response={
                    "errors": ['error converting input invalid for field "ttl"']
                },
            ),
        ]
    )
    def test_create_or_update_impersonated_account(
        self,
        method,
        expected_status_code,
        ttl,
        raises=None,
        expected_response=None,
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/impersonated-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_IMPERSONATED_ACCOUNT_NAME,
            )
        )

        token_scopes = DEFAULT_TOKEN_SCOPES

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.create_or_update_impersonated_account(
                        name=TEST_IMPERSONATED_ACCOUNT_NAME,
                        service_account_email=TEST_SERVICE_ACCOUNT_EMAIL,
                        token_scopes=token_scopes,
                        ttl=ttl,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertIn(
                    member='error converting input invalid for field "ttl"',
                    container=str(cm.exception),
                )
            else:
                resp = self._json_adapter.create_or_update_impersonated_account(
                    name=TEST_IMPERSONATED_ACCOUNT_NAME,
                    service_account_email=TEST_SERVICE_ACCOUNT_EMAIL,
                    token_scopes=token_scopes,
                    ttl=ttl,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "service_account_email": TEST_SERVICE_ACCOUNT_EMAIL,
                        "service_account_project": TEST_PROJECT_ID,
                        "token_scopes": DEFAULT_TOKEN_SCOPES,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=404,
                raises=InvalidPath,
            ),
        ]
    )
    def test_read_impersonated_account(
        self,
        method,
        expected_status_code,
        raises=None,
        expected_response=None,
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/impersonated-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_IMPERSONATED_ACCOUNT_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.read_impersonated_account(
                        name=TEST_IMPERSONATED_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.read_impersonated_account(
                    name=TEST_IMPERSONATED_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(
                method="LIST",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "keys": ["impersonated-account-01", "impersonated-account-02"]
                    }
                },
            ),
            param(method="LIST", expected_status_code=404, raises=InvalidPath),
            param(method="LIST", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_list_impersonated_accounts(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/impersonated-accounts".format(
                mount_point=TEST_MOUNT_POINT,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.list_impersonated_accounts(
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.list_impersonated_accounts(
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)

    @parameterized.expand(
        [
            param(method="DELETE", expected_status_code=204),
            param(method="DELETE", expected_status_code=404, raises=InvalidPath),
            param(method="DELETE", expected_status_code=405, raises=UnexpectedError),
        ]
    )
    def test_delete_impersonated_account(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = (
            "http://localhost:8200/v1/{mount_point}/impersonated-account/{name}".format(
                mount_point=TEST_MOUNT_POINT,
                name=TEST_IMPERSONATED_ACCOUNT_NAME,
            )
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.delete_impersonated_account(
                        name=TEST_IMPERSONATED_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.delete_impersonated_account(
                    name=TEST_IMPERSONATED_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp.status_code, expected_status_code)
                self.assertTrue(len(resp.content) == 0)

    @parameterized.expand(
        [
            param(
                method="GET",
                expected_status_code=200,
                expected_response={
                    "data": {
                        "expires_at_seconds": 1679109162,
                        "token": "ya29.c.b0Aaekm1Le-n2NCqrzZjdMtjpbgRji2yhiJkO...",
                        "token_ttl": 3598,
                    }
                },
            ),
            param(
                method="GET",
                expected_status_code=400,
                raises=InvalidRequest,
                expected_response={
                    "errors": [
                        'impersonated account "missing-account" does not exists'
                    ],
                },
            ),
        ]
    )
    def test_generate_impersonated_account_oauth2_access_token(
        self, method, expected_status_code, raises=None, expected_response=None
    ):
        mock_url = "http://localhost:8200/v1/{mount_point}/impersonated-account/{name}/token".format(
            mount_point=TEST_MOUNT_POINT,
            name=TEST_IMPERSONATED_ACCOUNT_NAME,
        )

        with requests_mock.mock() as requests_mocker:
            requests_mocker.register_uri(
                method=method,
                url=mock_url,
                status_code=expected_status_code,
                json=expected_response,
                headers={"Content-Type": "application/json"},
            )

            if raises:
                with self.assertRaises(raises) as cm:
                    self._json_adapter.generate_impersonated_account_oauth2_access_token(
                        name=TEST_IMPERSONATED_ACCOUNT_NAME,
                        mount_point=TEST_MOUNT_POINT,
                    )

                self.assertEqual(cm.exception.json, expected_response)
            else:
                resp = self._json_adapter.generate_impersonated_account_oauth2_access_token(
                    name=TEST_IMPERSONATED_ACCOUNT_NAME,
                    mount_point=TEST_MOUNT_POINT,
                )

                self.assertEqual(resp, expected_response)



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac import Client


class TestGcpMethods(TestCase):
    """Unit tests providing coverage for GCP auth backend-related methods/routes."""

    @parameterized.expand(
        [
            (
                "default mount point",
                "custom_role",
                "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9",
                None,
            ),
            (
                "custom mount point",
                "custom_role",
                "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9",
                "gcp-not-default",
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_auth_gcp(
        self, test_label, test_role, test_jwt, mount_point, requests_mocker
    ):
        mock_response = {
            "auth": {
                "accessor": "accessor-1234-5678-9012-345678901234",
                "client_token": "cltoken-1234-5678-9012-345678901234",
                "lease_duration": 10000,
                "metadata": {
                    "role": "custom_role",
                    "service_account_email": "dev1@project-123456.iam.gserviceaccount.com",
                    "service_account_id": "111111111111111111111",
                },
                "policies": ["default", "custom_role"],
                "renewable": True,
            },
            "data": None,
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "requesti-1234-5678-9012-345678901234",
            "warnings": [],
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/login".format(
            "gcp" if mount_point is None else mount_point
        )
        requests_mocker.register_uri(method="POST", url=mock_url, json=mock_response)
        client = Client()

        if mount_point is None:
            actual_response = client.auth.gcp.login(role=test_role, jwt=test_jwt)
        else:
            actual_response = client.auth.gcp.login(
                role=test_role, jwt=test_jwt, mount_point=mount_point
            )

        # ensure we received our mock response data back successfully
        self.assertEqual(mock_response, actual_response)



from authlib.integrations.flask_oauth2 import AuthorizationServer, ResourceProtector
from authlib.integrations.sqla_oauth2 import (
    create_bearer_token_validator,
    create_query_client_func,
    create_save_token_func,
)
from authlib.oauth2.rfc6749.grants import (
    AuthorizationCodeGrant as _AuthorizationCodeGrant,
)
from authlib.oidc.core import UserInfo
from authlib.oidc.core.grants import OpenIDCode as _OpenIDCode
from authlib.oidc.core.grants import OpenIDHybridGrant as _OpenIDHybridGrant
from authlib.oidc.core.grants import OpenIDImplicitGrant as _OpenIDImplicitGrant
from werkzeug.security import gen_salt
from tests.utils import get_config_file_path
from tests.utils.mock_oauth_provider.models import (
    OAuth2AuthorizationCode,
    OAuth2Client,
    OAuth2Token,
    User,
    db,
)

JWT_CONFIG = {
    "key": "".join(open(get_config_file_path("oidc_private.pem")).readlines()),
    "alg": "RS256",
    "exp": 3600,
}


def exists_nonce(nonce, req):
    exists = OAuth2AuthorizationCode.query.filter_by(
        client_id=req.client_id, nonce=nonce
    ).first()
    return bool(exists)


def generate_user_info(user, scope):
    return UserInfo(sub=str(user.id), name=user.username)


def create_authorization_code(client, grant_user, request):
    code = gen_salt(48)
    nonce = request.data.get("nonce")
    item = OAuth2AuthorizationCode(
        code=code,
        client_id=client.client_id,
        redirect_uri=request.redirect_uri,
        scope=request.scope,
        user_id=grant_user.id,
        nonce=nonce,
    )
    db.session.add(item)
    db.session.commit()
    return code


class AuthorizationCodeGrant(_AuthorizationCodeGrant):
    def generate_authorization_code(self):
        return gen_salt(48)

    def save_authorization_code(self, code, request):
        nonce = request.data.get("nonce")
        client = request.client
        item = OAuth2AuthorizationCode(
            code=code,
            client_id=client.client_id,
            redirect_uri=request.redirect_uri,
            scope=request.scope,
            user_id=request.user.id,
            nonce=nonce,
        )
        db.session.add(item)
        db.session.commit()
        return code

    def query_authorization_code(self, code, client):
        item = OAuth2AuthorizationCode.query.filter_by(
            code=code, client_id=client.client_id
        ).first()
        if item and not item.is_expired():
            return item

    def delete_authorization_code(self, authorization_code):
        db.session.delete(authorization_code)
        db.session.commit()

    def authenticate_user(self, authorization_code):
        return User.query.get(authorization_code.user_id)

    def get_jwt_config(self, grant):
        return JWT_CONFIG


class OpenIDCode(_OpenIDCode):
    def exists_nonce(self, nonce, request):
        return exists_nonce(nonce, request)

    def get_jwt_config(self, grant):
        return JWT_CONFIG

    def generate_user_info(self, user, scope):
        return generate_user_info(user, scope)


class ImplicitGrant(_OpenIDImplicitGrant):
    def exists_nonce(self, nonce, request):
        return exists_nonce(nonce, request)

    def get_jwt_config(self, grant):
        return self.jwt_config

    def generate_user_info(self, user, scope):
        return generate_user_info(user, scope)


class HybridGrant(_OpenIDHybridGrant):
    def create_authorization_code(self, client, grant_user, request):
        return create_authorization_code(client, grant_user, request)

    def exists_nonce(self, nonce, request):
        return exists_nonce(nonce, request)

    def get_jwt_config(self, grant):
        return JWT_CONFIG

    def generate_user_info(self, user, scope):
        return generate_user_info(user, scope)


authorization = AuthorizationServer()
require_oauth = ResourceProtector()


def config_oauth(app):
    global JWT_CONFIG
    JWT_CONFIG["iss"] = app.config.get("OAUTH2_JWT_ISS")
    query_client = create_query_client_func(db.session, OAuth2Client)
    save_token = create_save_token_func(db.session, OAuth2Token)
    authorization.init_app(app, query_client=query_client, save_token=save_token)

    # support all openid grants
    authorization.register_grant(
        AuthorizationCodeGrant,
        [
            OpenIDCode(require_nonce=True),
        ],
    )
    authorization.register_grant(ImplicitGrant)
    authorization.register_grant(HybridGrant)

    # protect resource
    bearer_cls = create_bearer_token_validator(db.session, OAuth2Token)
    require_oauth.register_token_validator(bearer_cls())



"""
Misc utility functions and constants
"""

import functools
import inspect
import os
import warnings
from textwrap import dedent
import urllib

from hvac import exceptions


def raise_for_error(
    method, url, status_code, message=None, errors=None, text=None, json=None
):
    """Helper method to raise exceptions based on the status code of a response received back from Vault.

    :param method: HTTP method of a request to Vault.
    :type method: str
    :param url: URL of the endpoint requested in Vault.
    :type url: str
    :param status_code: Status code received in a response from Vault.
    :type status_code: int
    :param message: Optional message to include in a resulting exception.
    :type message: str
    :param errors: Optional errors to include in a resulting exception.
    :type errors: list | str
    :param text: Optional text of the response.
    :type text: str
    :param json: Optional deserialized version of a JSON response (object)
    :type json: object

    :raises: hvac.exceptions.InvalidRequest | hvac.exceptions.Unauthorized | hvac.exceptions.Forbidden |
        hvac.exceptions.InvalidPath | hvac.exceptions.RateLimitExceeded | hvac.exceptions.InternalServerError |
        hvac.exceptions.VaultNotInitialized | hvac.exceptions.BadGateway | hvac.exceptions.VaultDown |
        hvac.exceptions.UnexpectedError

    """
    raise exceptions.VaultError.from_status(
        status_code,
        message,
        errors=errors,
        method=method,
        url=url,
        text=text,
        json=json,
    )


def aliased_parameter(
    name, *aliases, removed_in_version, position=None, raise_on_multiple=True
):
    """A decorator that can be used to define one or more aliases for a parameter,
    and optionally display a deprecation warning when aliases are used.
    It can also optionally raise an exception if a value is supplied via multiple names.
    LIMITATIONS:
    If the canonical parameter can be specified unnamed (positionally),
    then its position must be set to correctly detect multiple use and apply precedence.
    To set multiple aliases with different values for the optional parameters, use the decorator multiple times with the same name.
    This method will only work properly when the alias parameter is set as a keyword (named) arg, therefore the function in question
    should ensure that any aliases come after \\*args or bare \\* (marking keyword-only arguments: https://peps.python.org/pep-3102/).
    Note also that aliases do not have to appear in the original function's argument list.

    :param name: The canonical name of the parameter.
    :type name: str
    :param aliases: One or more alias names for the parameter.
    :type aliases: str
    :param removed_in_version: The version in which the alias will be removed. This should typically have a value.
        In the rare case that an alias is not deprecated, set this to None.
    :type removed_in_version: str | None
    :param position: The 0-based position of the canonical argument if it could be specified positionally. Use None for a keyword-only (named) argument.
    :type position: int
    :param raise_on_multiple: When True (default), raise an exception if a value is supplied via multiple names.
    :type raise_on_multiple: bool
    """

    def decorator(method):
        @functools.wraps(method)
        def wrapper(*args, **kwargs):
            has_canonical = False
            try:
                kwargs[name]
            except KeyError:
                if position is not None:
                    try:
                        args[position]
                    except IndexError:
                        pass
                    else:
                        has_canonical = True
            else:
                has_canonical = True

            # At this point if has_canonical is True, we'll never use an alias value,
            # but we're still looping so we can catch duplicates or deprecated aliases.
            for alias in aliases:
                if alias in kwargs:
                    # do deprecation before (potentially) raising on a duplicate to aid the user in choosing the right parameter.
                    if removed_in_version is not None:
                        deprecation_message = generate_parameter_deprecation_message(
                            to_be_removed_in_version=removed_in_version,
                            old_parameter_name=alias,
                            new_parameter_name=name,
                        )
                        warnings.warn(
                            message=deprecation_message,
                            category=DeprecationWarning,
                            stacklevel=2,
                        )

                    if not (has_canonical or name in kwargs):
                        kwargs[name] = kwargs[alias]
                    else:
                        if raise_on_multiple:
                            raise ValueError(
                                f"Parameter '{name}' was given a duplicate value via alias '{alias}'."
                            )

                    del kwargs[alias]

            return method(*args, **kwargs)

        return wrapper

    return decorator


def generate_parameter_deprecation_message(
    to_be_removed_in_version,
    old_parameter_name,
    new_parameter_name=None,
    extra_notes=None,
):
    """Generate a message to be used when warning about the use of deprecated paramers.

    :param to_be_removed_in_version: Version of this module the deprecated parameter will be removed in.
    :type to_be_removed_in_version: str
    :param old_parameter_name: Deprecated parameter name.
    :type old_parameter_name: str
    :param new_parameter_name: Parameter intended to replace the deprecated parameter, if applicable.
    :type new_parameter_name: str | None
    :param extra_notes: Optional freeform text used to provide additional context, alternatives, or notes.
    :type extra_notes: str | None
    :return: Full deprecation warning message for the indicated parameter.
    :rtype: str
    """

    message = f"Value supplied for deprecated parameter '{old_parameter_name}'. This parameter will be removed in version '{to_be_removed_in_version}'."
    if new_parameter_name is not None:
        message += f" Please use the '{new_parameter_name}' parameter moving forward."
    if extra_notes is not None:
        message += f" {extra_notes}"

    return message


def generate_method_deprecation_message(
    to_be_removed_in_version, old_method_name, method_name=None, module_name=None
):
    """Generate a message to be used when warning about the use of deprecated methods.

    :param to_be_removed_in_version: Version of this module the deprecated method will be removed in.
    :type to_be_removed_in_version: str
    :param old_method_name: Deprecated method name.
    :type old_method_name:  str
    :param method_name:  Method intended to replace the deprecated method indicated. This method's docstrings are
        included in the decorated method's docstring.
    :type method_name: str
    :param module_name: Name of the module containing the new method to use.
    :type module_name: str
    :return: Full deprecation warning message for the indicated method.
    :rtype: str
    """
    message = "Call to deprecated function '{old_method_name}'. This method will be removed in version '{version}'".format(
        old_method_name=old_method_name,
        version=to_be_removed_in_version,
    )
    if method_name is not None and module_name is not None:
        message += " Please use the '{method_name}' method on the '{module_name}' class moving forward.".format(
            method_name=method_name,
            module_name=module_name,
        )
    return message


def generate_property_deprecation_message(
    to_be_removed_in_version, old_name, new_name, new_attribute, module_name="Client"
):
    """Generate a message to be used when warning about the use of deprecated properties.

    :param to_be_removed_in_version: Version of this module the deprecated property will be removed in.
    :type to_be_removed_in_version: str
    :param old_name: Deprecated property name.
    :type old_name: str
    :param new_name: Name of the new property name to use.
    :type new_name: str
    :param new_attribute: The new attribute where the new property can be found.
    :type new_attribute: str
    :param module_name: Name of the module containing the new method to use.
    :type module_name: str
    :return: Full deprecation warning message for the indicated property.
    :rtype: str
    """
    message = "Call to deprecated property '{name}'. This property will be removed in version '{version}'".format(
        name=old_name,
        version=to_be_removed_in_version,
    )
    message += " Please use the '{new_name}' property on the '{module_name}.{new_attribute}' attribute moving forward.".format(
        new_name=new_name,
        module_name=module_name,
        new_attribute=new_attribute,
    )
    return message


def getattr_with_deprecated_properties(obj, item, deprecated_properties):
    """Helper method to use in the getattr method of a class with deprecated properties.

    :param obj: Instance of the Class containing the deprecated properties in question.
    :type obj: object
    :param item: Name of the attribute being requested.
    :type item: str
    :param deprecated_properties: List of deprecated properties. Each item in the list is a dict with at least a
        "to_be_removed_in_version" and "client_property" key to be used in the displayed deprecation warning.
    :type deprecated_properties: List[dict]
    :return: The new property indicated where available.
    :rtype: object
    """
    if item in deprecated_properties:
        deprecation_message = generate_property_deprecation_message(
            to_be_removed_in_version=deprecated_properties[item][
                "to_be_removed_in_version"
            ],
            old_name=item,
            new_name=deprecated_properties[item].get("new_property", item),
            new_attribute=deprecated_properties[item]["client_property"],
        )
        warnings.warn(
            message=deprecation_message,
            category=DeprecationWarning,
            stacklevel=2,
        )
        client_property = getattr(obj, deprecated_properties[item]["client_property"])
        return getattr(
            client_property, deprecated_properties[item].get("new_property", item)
        )

    raise AttributeError(
        "'{class_name}' has no attribute '{item}'".format(
            class_name=obj.__class__.__name__,
            item=item,
        )
    )


def deprecated_method(to_be_removed_in_version, new_method=None):
    """This is a decorator which can be used to mark methods as deprecated. It will result in a warning being emitted
    when the function is used.

    :param to_be_removed_in_version: Version of this module the decorated method will be removed in.
    :type to_be_removed_in_version: str
    :param new_method: Method intended to replace the decorated method. This method's docstrings are included in the
        decorated method's docstring.
    :type new_method: function
    :return: Wrapped function that includes a deprecation warning and update docstrings from the replacement method.
    :rtype: types.FunctionType
    """

    def decorator(method):
        if new_method is not None:
            new_method_name = new_method.__name__
            new_module_name = inspect.getmodule(new_method).__name__
        else:
            new_method_name, new_module_name = (None, None)

        deprecation_message = generate_method_deprecation_message(
            to_be_removed_in_version=to_be_removed_in_version,
            old_method_name=method.__name__,
            method_name=new_method_name,
            module_name=new_module_name,
        )

        @functools.wraps(method)
        def new_func(*args, **kwargs):
            warnings.warn(
                message=deprecation_message,
                category=DeprecationWarning,
                stacklevel=2,
            )
            return method(*args, **kwargs)

        if new_method:
            # Here we copy the docstring from the specified replacement method (i.e., the method to be used in place of
            # the one we're marking as deprecated) where available to set within the deprecated method's docstring.
            # If the "new" method has no docstring, we use a value of "N/A".
            docstring_copy = (
                new_method.__doc__ if new_method.__doc__ is not None else "N/A"
            )
            if new_method.__doc__ is not None:
                new_func.__doc__ = """\
                    {message}
                    Docstring content from this method's replacement copied below:
                    {docstring_copy}
                    """.format(
                    message=deprecation_message,
                    docstring_copy=dedent(docstring_copy),
                )

        else:
            new_func.__doc__ = deprecation_message
        return new_func

    return decorator


def validate_list_of_strings_param(param_name, param_argument):
    """Validate that an argument is a list of strings.

    :param param_name: The name of the parameter being validated. Used in any resulting exception messages.
    :type param_name: str | unicode
    :param param_argument: The argument to validate.
    :type param_argument: list
    :return: True if the argument is validated, False otherwise.
    :rtype: bool
    """
    if param_argument is None:
        param_argument = []
    if isinstance(param_argument, str):
        param_argument = param_argument.split(",")
    if not isinstance(param_argument, list) or not all(
        isinstance(p, str) for p in param_argument
    ):
        error_msg = 'unsupported {param} argument provided "{arg}" ({arg_type}), required type: List[str]'
        raise exceptions.ParamValidationError(
            error_msg.format(
                param=param_name,
                arg=param_argument,
                arg_type=type(param_argument),
            )
        )


def list_to_comma_delimited(list_param):
    """Convert a list of strings into a comma-delimited list / string.

    :param list_param: A list of strings.
    :type list_param: list
    :return: Comma-delimited string.
    :rtype: str
    """
    if list_param is None:
        list_param = []
    return ",".join(list_param)


def get_token_from_env():
    """Get the token from env var, VAULT_TOKEN. If not set, attempt to get the token from, ~/.vault-token

    :return: The vault token if set, else None
    :rtype: str | None
    """
    token = os.getenv("VAULT_TOKEN")
    if not token:
        token_file_path = os.path.expanduser("~/.vault-token")
        if os.path.exists(token_file_path):
            with open(token_file_path) as f_in:
                token = f_in.read().strip()

    if not token:
        return None

    return token


def comma_delimited_to_list(list_param):
    """Convert comma-delimited list / string into a list of strings

    :param list_param: Comma-delimited string
    :type list_param: str | unicode
    :return: A list of strings
    :rtype: list
    """
    if isinstance(list_param, list):
        return list_param
    if isinstance(list_param, str):
        return list_param.split(",")
    else:
        return []


def validate_pem_format(param_name, param_argument):
    """Validate that an argument is a PEM-formatted public key or certificate

    :param param_name: The name of the parameter being validate. Used in any resulting exception messages.
    :type param_name: str | unicode
    :param param_argument: The argument to validate
    :type param_argument: str | unicode
    :return: True if the argument is validate False otherwise
    :rtype: bool
    """

    def _check_pem(arg):
        arg = arg.strip()
        if not arg.startswith("-----BEGIN CERTIFICATE-----") or not arg.endswith(
            "-----END CERTIFICATE-----"
        ):
            return False
        return True

    if isinstance(param_argument, str):
        param_argument = [param_argument]

    if not isinstance(param_argument, list) or not all(
        _check_pem(p) for p in param_argument
    ):
        error_msg = (
            "unsupported {param} public key / certificate format, required type: PEM"
        )
        raise exceptions.ParamValidationError(error_msg.format(param=param_name))


def remove_nones(params):
    """Removes None values from optional arguments in a parameter dictionary.

    :param params: The dictionary of parameters to be filtered.
    :type params: dict
    :return: A filtered copy of the parameter dictionary.
    :rtype: dict
    """

    return {key: value for key, value in params.items() if value is not None}


def format_url(format_str, *args, **kwargs):
    """Creates a URL using the specified format after escaping the provided arguments.

    :param format_str: The URL containing replacement fields.
    :type format_str: str
    :param kwargs: Positional replacement field values.
    :type kwargs: list
    :param kwargs: Named replacement field values.
    :type kwargs: dict
    :return: The formatted URL path with escaped replacement fields.
    :rtype: str
    """

    def url_quote(maybe_str):
        # Special care must be taken for Python 2 where Unicode characters will break urllib quoting.
        # To work around this, we always cast to a Unicode type, then UTF-8 encode it.
        # Doing this is version agnostic and returns the same result in Python 2 or 3.
        unicode_str = str(maybe_str)
        utf8_str = unicode_str.encode("utf-8")
        return urllib.parse.quote(utf8_str)

    escaped_args = [url_quote(value) for value in args]
    escaped_kwargs = {key: url_quote(value) for key, value in kwargs.items()}

    return format_str.format(*escaped_args, **escaped_kwargs)



#!/usr/bin/env python
import logging
import os
from time import sleep

from requests_mock.mocker import Mocker

from tests import utils as test_utils
from tests.utils.mock_ldap_server import MockLdapServer
from tests.utils.server_manager import ServerManager


def doctest_global_setup():
    client = test_utils.create_client()
    manager = ServerManager(
        config_paths=[test_utils.get_config_file_path("vault-doctest.hcl")],
        client=client,
    )
    manager.start()
    manager.initialize()
    manager.unseal()

    mocker = Mocker(real_http=True)
    mocker.start()

    auth_method_paths = [
        f"ldap/login/{MockLdapServer.ldap_user_name}",
    ]
    for auth_method_path in auth_method_paths:
        mock_url = f"https://127.0.0.1:8200/v1/auth/{auth_method_path}"
        mock_response = {
            "auth": {
                "client_token": manager.root_token,
                "accessor": "0e9e354a-520f-df04-6867-ee81cae3d42d",
                "policies": ["default"],
                "lease_duration": 2764800,
                "renewable": True,
            },
        }
        mocker.register_uri(
            method="POST",
            url=mock_url,
            json=mock_response,
        )

    client.token = manager.root_token
    os.environ["VAULT_TOKEN"] = manager.root_token
    os.environ["REQUESTS_CA_BUNDLE"] = test_utils.get_config_file_path(
        "server-cert.pem"
    )
    os.environ["LDAP_USERNAME"] = MockLdapServer.ldap_user_name
    os.environ["LDAP_PASSWORD"] = MockLdapServer.ldap_user_password
    os.environ["AWS_LAMBDA_FUNCTION_NAME"] = "hvac-lambda"
    os.environ.setdefault("LDAP_PASSWORD", MockLdapServer.ldap_user_password)

    if "secret/" not in client.sys.list_mounted_secrets_engines()["data"]:
        client.sys.enable_secrets_engine(
            backend_type="kv",
            path="secret",
            options=dict(version=2),
        )
        attempts = 0
        while (
            attempts < 25
            and "secret/" not in client.sys.list_mounted_secrets_engines()["data"]
        ):
            attempts += 1
            logging.debug(
                "Waiting 1 second for KV V2 secrets engine under path {path} to become available...".format(
                    path="secret",
                )
            )
            sleep(1)

    return manager, mocker



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.auth_methods import Github
from hvac.api.auth_methods.github import DEFAULT_MOUNT_POINT


class TestGithub(TestCase):
    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_configure(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        github = Github(adapter=JSONAdapter())
        response = github.configure(
            organization="hvac",
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_configuration(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "auth": None,
            "data": {"base_url": "", "max_ttl": 0, "organization": "", "ttl": 0},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "860a11a8-b835-cbab-7fce-de4edc4cf533",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        github = Github(adapter=JSONAdapter())
        response = github.read_configuration(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_map_team(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        team_name = "hvac"
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/map/teams/{team_name}".format(
                mount_point=mount_point,
                team_name=team_name,
            )
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        github = Github(adapter=JSONAdapter())
        response = github.map_team(
            team_name=team_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_team_mapping(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        team_name = "hvac"
        mock_response = {
            "auth": None,
            "data": {"key": "SOME_TEAM", "value": "some-team-policy"},
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "50346cc8-34e7-f2ea-f36a-fcb9d45c1676",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/map/teams/{team_name}".format(
                mount_point=mount_point,
                team_name=team_name,
            )
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        github = Github(adapter=JSONAdapter())
        response = github.read_team_mapping(
            team_name=team_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_map_user(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        user_name = "hvac"
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/map/users/{user_name}".format(
                mount_point=mount_point,
                user_name=user_name,
            )
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        github = Github(adapter=JSONAdapter())
        response = github.map_user(
            user_name=user_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_user_mapping(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        user_name = "hvac"
        mock_response = {
            "auth": None,
            "data": None,
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "71ec6e1b-6d4e-6374-ddc2-ff1cdd860e60",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/map/users/{user_name}".format(
                mount_point=mount_point,
                user_name=user_name,
            )
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        github = Github(adapter=JSONAdapter())
        response = github.read_user_mapping(
            user_name=user_name,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_login(self, test_label, mount_point, requests_mocker):
        mock_response = {
            "auth": {
                "accessor": "f578d442-94ec-11e8-afe4-0af6a65f93f6",
                "client_token": "edf5c2c0-94ec-11e8-afe4-0af6a65f93f6",
                "entity_id": "f9268760-94ec-11e8-afe4-0af6a65f93f6",
                "lease_duration": 3600,
                "metadata": {"org": "hvac", "username": "hvacbot"},
                "policies": [
                    "default",
                ],
                "renewable": True,
                "token_policies": ["default"],
            },
            "data": None,
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "488cf309-2f81-cc04-51bf-c43063d309eb",
            "warnings": None,
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/login".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            json=mock_response,
        )
        github = Github(adapter=JSONAdapter())
        response = github.login(
            token="valid-token",
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )
        self.assertEqual(
            first=mock_response["auth"]["client_token"],
            second=github._adapter.token,
        )



import logging
from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.auth_methods import Okta


class TestOkta(TestCase):
    TEST_MOUNT_POINT = "okta-test"
    TEST_USERNAME = "hvac-person"

    @parameterized.expand(
        [
            (
                "success",
                dict(),
                None,
            ),
        ]
    )
    @requests_mock.Mocker()
    def test_login(self, label, test_params, raises, requests_mocker):
        test_policies = [
            "default",
        ]
        expected_status_code = 200
        mock_url = (
            "http://localhost:8200/v1/auth/{mount_point}/login/{username}".format(
                mount_point=self.TEST_MOUNT_POINT,
                username=self.TEST_USERNAME,
            )
        )
        mock_response = {
            "lease_id": "",
            "data": None,
            "warnings": None,
            "auth": {
                "client_token": "64d2a8f2-2a2f-5688-102b-e6088b76e344",
                "accessor": "18bb8f89-826a-56ee-c65b-1736dc5ea27d",
                "policies": ["default"],
                "metadata": {"username": self.TEST_USERNAME, "policies": "default"},
            },
            "lease_duration": 7200,
            "renewable": True,
        }
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        okta = Okta(adapter=JSONAdapter())
        if raises is not None:
            with self.assertRaises(raises):
                okta.login(
                    username=self.TEST_USERNAME,
                    password="badpassword",
                    mount_point=self.TEST_MOUNT_POINT,
                    **test_params
                )
        else:
            login_response = okta.login(
                username=self.TEST_USERNAME,
                password="badpassword",
                mount_point=self.TEST_MOUNT_POINT,
                **test_params
            )
            logging.debug("login_response: %s" % login_response)
            self.assertEqual(
                first=login_response["auth"]["policies"],
                second=test_policies,
            )



from unittest import TestCase

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestLeader(HvacIntegrationTestCase, TestCase):
    def test_read_health_status(self):
        self.assertIn(
            member="ha_enabled",
            container=self.client.sys.read_leader_status(),
        )



import os
from flask import Flask
from tests.utils.mock_oauth_provider.models import db
from tests.utils.mock_oauth_provider.oauth2 import config_oauth
from tests.utils.mock_oauth_provider.routes import bp


def create_app(config=None):
    app = Flask(__name__)

    # load default configuration
    app.config.from_object("tests.utils.mock_oauth_provider.settings")

    # load environment configuration
    if "WEBSITE_CONF" in os.environ:
        app.config.from_envvar("WEBSITE_CONF")

    # load app sepcified configuration
    if config is not None:
        if isinstance(config, dict):
            app.config.update(config)
        elif config.endswith(".py"):
            app.config.from_pyfile(config)

    setup_app(app)
    return app


def setup_app(app):
    # Create tables if they do not exist already
    @app.before_first_request
    def create_tables():
        db.create_all()

    db.init_app(app)
    config_oauth(app)
    app.register_blueprint(bp, url_prefix="")



from unittest import TestCase

from hvac import Client
from hvac.adapters import JSONAdapter


class TestClient(TestCase):
    """Unit tests providing coverage for client related methods."""

    def test_setting_adapter_on_client_sets_adapter_of_endpoint_classes(self):
        client = Client()
        old_adapter = client.adapter

        client.adapter = JSONAdapter()

        self.assertIsNot(client.adapter, old_adapter)
        self.assertSetEqual(
            {client.adapter},
            {client.secrets.adapter, client.sys.adapter, client.auth.adapter},
        )



#!/usr/bin/env python
"""LDAP methods module."""
from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "ldap"


class Ldap(VaultApiBase):
    """LDAP Auth Method (API).

    Reference: https://www.vaultproject.io/api/auth/ldap/index.html
    """

    @utils.aliased_parameter(
        "userdn", "user_dn", removed_in_version="3.0.0", position=1
    )
    @utils.aliased_parameter(
        "groupdn", "group_dn", removed_in_version="3.0.0", position=2
    )
    @utils.aliased_parameter(
        "binddn", "bind_dn", removed_in_version="3.0.0", position=10
    )
    @utils.aliased_parameter(
        "bindpass", "bind_pass", removed_in_version="3.0.0", position=11
    )
    @utils.aliased_parameter(
        "userattr", "user_attr", removed_in_version="3.0.0", position=12
    )
    @utils.aliased_parameter(
        "discoverdn", "discover_dn", removed_in_version="3.0.0", position=13
    )
    @utils.aliased_parameter(
        "upndomain", "upn_domain", removed_in_version="3.0.0", position=15
    )
    @utils.aliased_parameter(
        "groupfilter", "group_filter", removed_in_version="3.0.0", position=16
    )
    @utils.aliased_parameter(
        "groupattr", "group_attr", removed_in_version="3.0.0", position=17
    )
    def configure(
        self,
        userdn=None,
        groupdn=None,
        url=None,
        case_sensitive_names=None,
        starttls=None,
        tls_min_version=None,
        tls_max_version=None,
        insecure_tls=None,
        certificate=None,
        binddn=None,
        bindpass=None,
        userattr=None,
        discoverdn=None,
        deny_null_bind=True,
        upndomain=None,
        groupfilter=None,
        groupattr=None,
        use_token_groups=None,
        token_ttl=None,
        token_max_ttl=None,
        mount_point=DEFAULT_MOUNT_POINT,
        *,
        anonymous_group_search=None,
        client_tls_cert=None,
        client_tls_key=None,
        connection_timeout=None,
        dereference_aliases=None,
        max_page_size=None,
        request_timeout=None,
        token_bound_cidrs=None,
        token_explicit_max_ttl=None,
        token_no_default_policy=None,
        token_num_uses=None,
        token_period=None,
        token_policies=None,
        token_type=None,
        userfilter=None,
        username_as_alias=None,
    ):
        """
        Configure the LDAP auth method.

        Supported methods:
            POST: /auth/{mount_point}/config. Produces: 204 (empty body)

        :param anonymous_group_search: Use anonymous binds when performing LDAP group searches (note: even when true,
            the initial credentials will still be used for the initial connection test).
        :type anonymous_group_search: bool
        :param client_tls_cert: Client certificate to provide to the LDAP server, must be x509 PEM encoded.
        :type client_tls_cert: str | unicode
        :param client_tls_key: Client certificate key to provide to the LDAP server, must be x509 PEM encoded.
        :type client_tls_key: str | unicode
        :param connection_timeout: Timeout, in seconds, when attempting to connect to the LDAP server before trying the
            next URL in the configuration.
        :type connection_timeout: int
        :param dereference_aliases: When aliases should be dereferenced on search operations.
            Accepted values are 'never', 'finding', 'searching', 'always'.
        :type dereference_aliases: str | unicode
        :param max_page_size: If set to a value greater than 0, the LDAP backend will use the LDAP server's paged search
            control to request pages of up to the given size.
        :type max_page_size: int
        :param request_timeout: Timeout, in seconds, for the connection when making requests against the server before
            returning back an error.
        :type request_timeout: str | unicode
        :param token_bound_cidrs: List of CIDR blocks; if set, specifies blocks of IP addresses which can authenticate
            successfully, and ties the resulting token to these blocks as well.
        :type token_bound_cidrs: list
        :param token_explicit_max_ttl: If set, will encode an explicit max TTL onto the token. This is a hard cap even
            if token_ttl and token_max_ttl would otherwise allow a renewal.
        :type token_explicit_max_ttl: str | unicode
        :param token_no_default_policy: If set, the default policy will not be set on generated tokens; otherwise it
            will be added to the policies set in token_policies.
        :type token_no_default_policy: bool
        :param token_num_uses: The maximum number of times a generated token may be used (within its lifetime); 0 means
            unlimited.
        :type token_num_uses: int
        :param token_period: The maximum allowed period value when a periodic token is requested from this role.
        :type token_period: str | unicode
        :param token_policies: List of token policies to encode onto generated tokens.
        :type token_policies: list
        :param token_type: The type of token that should be generated.
        :type token_type: str | unicode
        :param userfilter: An optional LDAP user search filter.
        :type userfilter: str | unicode
        :param username_as_alias: If set to true, forces the auth method to use the username passed by the user as the
            alias name.
        :type username_as_alias: bool
        :param userdn: Base DN under which to perform user search. Example: ou=Users,dc=example,dc=com
        :type userdn: str | unicode
        :param user_dn: Alias for userdn. This alias will be removed in v3.0.0.
        :type user_dn: str | unicode
        :param groupdn: LDAP search base to use for group membership search. This can be the root containing either
            groups or users. Example: ou=Groups,dc=example,dc=com
        :type groupdn: str | unicode
        :param group_dn: Alias for groupdn. This alias will be removed in v3.0.0.
        :type group_dn: str | unicode
        :param url: The LDAP server to connect to. Examples: ldap://ldap.myorg.com, ldaps://ldap.myorg.com:636.
            Multiple URLs can be specified with commas, e.g. ldap://ldap.myorg.com,ldap://ldap2.myorg.com; these will be
            tried in-order.
        :type url: str | unicode
        :param case_sensitive_names: If set, user and group names assigned to policies within the backend will be case
            sensitive. Otherwise, names will be normalized to lower case. Case will still be preserved when sending the
            username to the LDAP server at login time; this is only for matching local user/group definitions.
        :type case_sensitive_names: bool
        :param starttls: If true, issues a StartTLS command after establishing an unencrypted connection.
        :type starttls: bool
        :param tls_min_version: Minimum TLS version to use. Accepted values are tls10, tls11 or tls12.
        :type tls_min_version: str | unicode
        :param tls_max_version: Maximum TLS version to use. Accepted values are tls10, tls11 or tls12.
        :type tls_max_version: str | unicode
        :param insecure_tls: If true, skips LDAP server SSL certificate verification - insecure, use with caution!
        :type insecure_tls: bool
        :param certificate: CA certificate to use when verifying LDAP server certificate, must be x509 PEM encoded.
        :type certificate: str | unicode
        :param binddn: Distinguished name of object to bind when performing user search. Example:
            cn=vault,ou=Users,dc=example,dc=com
        :type binddn: str | unicode
        :param bind_dn: Alias for binddn. This alias will be removed in v3.0.0.
        :type bind_dn: str | unicode
        :param bindpass:  Password to use along with binddn when performing user search.
        :type bindpass: str | unicode
        :param bind_pass: Alias for bindpass. This alias will be removed in v3.0.0.
        :type bind_pass: str | unicode
        :param userattr: Attribute on user attribute object matching the username passed when authenticating. Examples:
            sAMAccountName, cn, uid
        :type userattr: str | unicode
        :param user_attr: Alias for userattr. This alias will be removed in v3.0.0.
        :type user_attr: str | unicode
        :param discoverdn: Use anonymous bind to discover the bind DN of a user.
        :type discoverdn: bool
        :param discover_dn: Alias for discoverdn. This alias will be removed in v3.0.0.
        :type discover_dn: bool
        :param deny_null_bind: This option prevents users from bypassing authentication when providing an empty password.
        :type deny_null_bind: bool
        :param upndomain: The userPrincipalDomain used to construct the UPN string for the authenticating user. The
            constructed UPN will appear as [username]@UPNDomain. Example: example.com, which will cause vault to bind as
            username@example.com.
        :type upndomain: str | unicode
        :param upn_domain: Alias for upndomain. This alias will be removed in v3.0.0.
        :type upn_domain: str | unicode
        :param groupfilter: Go template used when constructing the group membership query. The template can access the
            following context variables: [UserDN, Username]. The default is
            `(|(memberUid={{.Username}})(member={{.UserDN}})(uniqueMember={{.UserDN}}))`, which is compatible with several
            common directory schemas. To support nested group resolution for Active Directory, instead use the following
            query: (&(objectClass=group)(member:1.2.840.113556.1.4.1941:={{.UserDN}})).
        :type groupfilter: str | unicode
        :param group_filter: Alias for groupfilter. This alias will be removed in v3.0.0.
        :type group_filter: str | unicode
        :param groupattr: LDAP attribute to follow on objects returned by groupfilter in order to enumerate user group
            membership. Examples: for groupfilter queries returning group objects, use: cn. For queries returning user
            objects, use: memberOf. The default is cn.
        :type groupattr: str | unicode
        :param group_attr: Alias for groupattr. This alias will be removed in v3.0.0.
        :type group_attr: str | unicode
        :param use_token_groups: If true, groups are resolved through Active Directory tokens. This may speed up nested
            group membership resolution in large directories.
        :type use_token_groups: bool
        :param token_ttl: The incremental lifetime for generated tokens.
        :type token_ttl: str | unicode
        :param token_max_ttl: The maximum lifetime for generated tokens.
        :type token_max_ttl: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the configure request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "url": url,
                "anonymous_group_search": anonymous_group_search,
                "binddn": binddn,
                "bindpass": bindpass,
                "case_sensitive_names": case_sensitive_names,
                "certificate": certificate,
                "client_tls_cert": client_tls_cert,
                "client_tls_key": client_tls_key,
                "connection_timeout": connection_timeout,
                "deny_null_bind": deny_null_bind,
                "dereference_aliases": dereference_aliases,
                "discoverdn": discoverdn,
                "groupattr": groupattr,
                "groupdn": groupdn,
                "groupfilter": groupfilter,
                "insecure_tls": insecure_tls,
                "max_page_size": max_page_size,
                "request_timeout": request_timeout,
                "starttls": starttls,
                "tls_max_version": tls_max_version,
                "tls_min_version": tls_min_version,
                "token_bound_cidrs": token_bound_cidrs,
                "token_explicit_max_ttl": token_explicit_max_ttl,
                "token_max_ttl": token_max_ttl,
                "token_no_default_policy": token_no_default_policy,
                "token_num_uses": token_num_uses,
                "token_period": token_period,
                "token_policies": token_policies,
                "token_ttl": token_ttl,
                "token_type": token_type,
                "upndomain": upndomain,
                "use_token_groups": use_token_groups,
                "userattr": userattr,
                "userdn": userdn,
                "userfilter": userfilter,
                "username_as_alias": username_as_alias,
            }
        )

        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_configuration(self, mount_point=DEFAULT_MOUNT_POINT):
        """
        Retrieve the LDAP configuration for the auth method.

        Supported methods:
            GET: /auth/{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_configuration request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.get(
            url=api_path,
        )

    def create_or_update_group(
        self, name, policies=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """
        Create or update LDAP group policies.

        Supported methods:
            POST: /auth/{mount_point}/groups/{name}. Produces: 204 (empty body)


        :param name: The name of the LDAP group
        :type name: str | unicode
        :param policies: List of policies associated with the group. This parameter is transformed to a comma-delimited
            string before being passed to Vault.
        :type policies: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_group request.
        :rtype: requests.Response
        """
        if policies is not None and not isinstance(policies, list):
            error_msg = '"policies" argument must be an instance of list or None, "{policies_type}" provided.'.format(
                policies_type=type(policies),
            )
            raise exceptions.ParamValidationError(error_msg)

        params = {}
        if policies is not None:
            params["policies"] = ",".join(policies)
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def list_groups(self, mount_point=DEFAULT_MOUNT_POINT):
        """
        List existing LDAP existing groups that have been created in this auth method.

        Supported methods:
            LIST: /auth/{mount_point}/groups. Produces: 200 application/json


        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the list_groups request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def read_group(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """
        Read policies associated with a LDAP group.

        Supported methods:
            GET: /auth/{mount_point}/groups/{name}. Produces: 200 application/json


        :param name: The name of the LDAP group
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_group request.
        :rtype: dict
        """
        params = {
            "name": name,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
            json=params,
        )

    def delete_group(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """
        Delete a LDAP group and policy association.

        Supported methods:
            DELETE: /auth/{mount_point}/groups/{name}. Produces: 204 (empty body)


        :param name: The name of the LDAP group
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_group request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def create_or_update_user(
        self, username, policies=None, groups=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """
        Create or update LDAP users policies and group associations.

        Supported methods:
            POST: /auth/{mount_point}/users/{username}. Produces: 204 (empty body)


        :param username: The username of the LDAP user
        :type username: str | unicode
        :param policies: List of policies associated with the user. This parameter is transformed to a comma-delimited
            string before being passed to Vault.
        :type policies: str | unicode
        :param groups: List of groups associated with the user. This parameter is transformed to a comma-delimited
            string before being passed to Vault.
        :type groups: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the create_or_update_user request.
        :rtype: requests.Response
        """
        list_required_params = {
            "policies": policies,
            "groups": groups,
        }
        for param_name, param_arg in list_required_params.items():
            if param_arg is not None and not isinstance(param_arg, list):
                error_msg = '"{param_name}" argument must be an instance of list or None, "{param_type}" provided.'.format(
                    param_name=param_name,
                    param_type=type(param_arg),
                )
                raise exceptions.ParamValidationError(error_msg)

        params = {}
        if policies is not None:
            params["policies"] = ",".join(policies)
        if groups is not None:
            params["groups"] = ",".join(groups)
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def list_users(self, mount_point=DEFAULT_MOUNT_POINT):
        """
        List existing users in the method.

        Supported methods:
            LIST: /auth/{mount_point}/users. Produces: 200 application/json


        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the list_users request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def read_user(self, username, mount_point=DEFAULT_MOUNT_POINT):
        """
        Read policies associated with a LDAP user.

        Supported methods:
            GET: /auth/{mount_point}/users/{username}. Produces: 200 application/json


        :param username: The username of the LDAP user
        :type username: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_user request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.get(
            url=api_path,
        )

    def delete_user(self, username, mount_point=DEFAULT_MOUNT_POINT):
        """
        Delete a LDAP user and policy association.

        Supported methods:
            DELETE: /auth/{mount_point}/users/{username}. Produces: 204 (empty body)


        :param username: The username of the LDAP user
        :type username: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the delete_user request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def login(
        self, username, password, use_token=True, mount_point=DEFAULT_MOUNT_POINT
    ):
        """
        Log in with LDAP credentials.

        Supported methods:
            POST: /auth/{mount_point}/login/{username}. Produces: 200 application/json


        :param username: The username of the LDAP user
        :type username: str | unicode
        :param password: The password for the LDAP user
        :type password: str | unicode
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
        :type use_token: bool
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the login_with_user request.
        :rtype: requests.Response
        """
        params = {
            "password": password,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/login/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.login(
            url=api_path,
            use_token=use_token,
            json=params,
        )



#!/usr/bin/env python
import json
import logging
from unittest import TestCase

from parameterized import parameterized, param

from hvac.exceptions import ParamValidationError
from tests.utils import vault_version_lt
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestAws(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "aws-test"
    TEST_ROLE_NAME = "hvac-test-role"
    TEST_POLICY_DOCUMENT = {
        "Statement": [
            {"Action": "ec2:Describe*", "Effect": "Allow", "Resource": "*"},
        ],
        "Version": "2012-10-17",
    }

    def setUp(self):
        super().setUp()
        if "%s/" % self.TEST_MOUNT_POINT not in self.client.sys.list_auth_methods():
            self.client.sys.enable_secrets_engine(
                backend_type="aws",
                path=self.TEST_MOUNT_POINT,
            )

    def tearDown(self):
        self.client.sys.disable_secrets_engine(
            path=self.TEST_MOUNT_POINT,
        )
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_configure_root_iam_credentials(
        self, label, credentials="", raises=None, exception_message=""
    ):
        access_key = "butts"
        secret_key = "secret-butts"
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.configure_root_iam_credentials(
                    access_key=access_key,
                    secret_key=secret_key,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            configure_response = self.client.secrets.aws.configure_root_iam_credentials(
                access_key=access_key,
                secret_key=secret_key,
                iam_endpoint="localhost",
                sts_endpoint="localhost",
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_configure_lease(
        self, label, lease="60s", lease_max="120s", raises=None, exception_message=""
    ):
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.configure_lease(
                    lease=lease,
                    lease_max=lease_max,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            configure_response = self.client.secrets.aws.configure_lease(
                lease=lease,
                lease_max=lease_max,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)
            self.assertEqual(
                first=bool(configure_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_lease(
        self,
        label,
        lease="60s",
        lease_max="120s",
        configure_first=True,
        raises=None,
        exception_message="",
    ):
        if configure_first:
            configure_response = self.client.secrets.aws.configure_lease(
                lease=lease,
                lease_max=lease_max,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("configure_response: %s" % configure_response)

        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.read_lease_config(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            read_response = self.client.secrets.aws.read_lease_config(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_response: %s" % read_response)
            self.assertEqual(
                first=int(lease_max.replace("s", "")),
                second=self.convert_python_ttl_value_to_expected_vault_response(
                    ttl_value=read_response["data"]["lease_max"],
                ),
            )

    @parameterized.expand(
        [
            param(
                "success",
                policy_document={
                    "Statement": [
                        {"Action": "ec2:Describe*", "Effect": "Allow", "Resource": "*"},
                    ],
                    "Version": "2012-10-17",
                },
            ),
            param(
                "with policy_arns",
                policy_arns=["arn:aws:iam::aws:policy/AmazonVPCReadOnlyAccess"],
            ),
            param(
                "assumed_role with policy document",
                policy_document={
                    "Statement": [
                        {"Action": "ec2:Describe*", "Effect": "Allow", "Resource": "*"},
                    ],
                    "Version": "2012-10-17",
                },
                credential_type="assumed_role",
            ),
            param(
                "invalid credential type",
                credential_type="cat",
                raises=ParamValidationError,
                exception_message="invalid credential_type argument provided",
            ),
        ]
    )
    def test_create_or_update_role(
        self,
        label,
        credential_type="iam_user",
        policy_document=None,
        default_sts_ttl=None,
        max_sts_ttl=None,
        role_arns=None,
        policy_arns=None,
        raises=None,
        exception_message="",
    ):
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.create_or_update_role(
                    name=self.TEST_ROLE_NAME,
                    credential_type=credential_type,
                    policy_document=policy_document,
                    default_sts_ttl=default_sts_ttl,
                    max_sts_ttl=max_sts_ttl,
                    role_arns=role_arns,
                    policy_arns=policy_arns,
                    legacy_params=vault_version_lt("0.11.0"),
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            role_response = self.client.secrets.aws.create_or_update_role(
                name=self.TEST_ROLE_NAME,
                credential_type=credential_type,
                policy_document=policy_document,
                default_sts_ttl=default_sts_ttl,
                max_sts_ttl=max_sts_ttl,
                role_arns=role_arns,
                policy_arns=policy_arns,
                legacy_params=vault_version_lt("0.11.0"),
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("role_response: %s" % role_response)

            self.assertEqual(
                first=bool(role_response),
                second=True,
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_read_role(
        self, label, configure_first=True, raises=None, exception_message=""
    ):
        if configure_first:
            self.client.secrets.aws.create_or_update_role(
                name=self.TEST_ROLE_NAME,
                credential_type="iam_user",
                policy_document=self.TEST_POLICY_DOCUMENT,
                legacy_params=vault_version_lt("0.11.0"),
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.read_role(
                    name=self.TEST_ROLE_NAME,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            read_role_response = self.client.secrets.aws.read_role(
                name=self.TEST_ROLE_NAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("read_role_response: %s" % read_role_response)
            if vault_version_lt("0.11.0"):
                self.assertDictEqual(
                    d1=json.loads(read_role_response["data"]["policy"]),
                    d2=self.TEST_POLICY_DOCUMENT,
                )
            # https://github.com/hashicorp/vault/commit/2dcd0aed2a242f53dae03318b4d68693f7d92b81
            elif vault_version_lt("1.0.2"):
                self.assertEqual(
                    first=read_role_response["data"]["credential_types"],
                    second=["iam_user"],
                )
            else:
                self.assertEqual(
                    first=read_role_response["data"]["credential_type"],
                    second="iam_user",
                )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_list_roles(
        self, label, configure_first=True, raises=None, exception_message=""
    ):
        if configure_first:
            self.client.secrets.aws.create_or_update_role(
                name=self.TEST_ROLE_NAME,
                credential_type="iam_user",
                policy_document=self.TEST_POLICY_DOCUMENT,
                legacy_params=vault_version_lt("0.11.0"),
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.list_roles(
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            list_roles_response = self.client.secrets.aws.list_roles(
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("list_roles_response: %s" % list_roles_response)
            self.assertEqual(
                first=list_roles_response["data"]["keys"],
                second=[self.TEST_ROLE_NAME],
            )

    @parameterized.expand(
        [
            param(
                "success",
            ),
        ]
    )
    def test_delete_role(
        self, label, configure_first=True, raises=None, exception_message=""
    ):
        if configure_first:
            self.client.secrets.aws.create_or_update_role(
                name=self.TEST_ROLE_NAME,
                credential_type="iam_user",
                policy_document=self.TEST_POLICY_DOCUMENT,
                legacy_params=vault_version_lt("0.11.0"),
                mount_point=self.TEST_MOUNT_POINT,
            )
        if raises:
            with self.assertRaises(raises) as cm:
                self.client.secrets.aws.delete_role(
                    name=self.TEST_ROLE_NAME,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(
                member=exception_message,
                container=str(cm.exception),
            )
        else:
            delete_role_response = self.client.secrets.aws.delete_role(
                name=self.TEST_ROLE_NAME,
                mount_point=self.TEST_MOUNT_POINT,
            )
            logging.debug("delete_role_response: %s" % delete_role_response)
            self.assertEqual(
                first=bool(delete_role_response),
                second=True,
            )



import logging
from unittest import TestCase

from parameterized import parameterized, param
from tests.utils import create_client
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestHealth(HvacIntegrationTestCase, TestCase):
    enable_vault_ha = True

    def tearDown(self):
        # If one of our test cases left the Vault cluster sealed, unseal it here.
        self.manager.unseal()
        super().tearDown()

    @parameterized.expand(
        [
            param(
                "default params",
            ),
            param(
                "unsealed standby node HEAD method",
                use_standby_node=True,
                method="HEAD",
                expected_status_code=429,
                ha_required=True,
            ),
            param(
                "unsealed standby node GET method",
                use_standby_node=True,
                method="GET",
                expected_status_code=429,
                ha_required=True,
            ),
            param(
                "sealed standby node HEAD method",
                use_standby_node=True,
                method="HEAD",
                expected_status_code=503,
                seal_first=True,
                ha_required=True,
            ),
            param(
                "sealed standby node GET method",
                use_standby_node=True,
                method="GET",
                expected_status_code=503,
                seal_first=True,
                ha_required=True,
            ),
            param("GET method", method="GET"),
        ]
    )
    def test_read_health_status(
        self,
        label,
        method="HEAD",
        use_standby_node=False,
        expected_status_code=200,
        seal_first=False,
        ha_required=False,
    ):
        """Test the Health system backend class's "read_health_status" method.

        :param label: Label for a given parameterized test case.
        :type label: str
        :param method: HTTP method to use when invoking the method under test (GET or HEAD available).
        :type method: str
        :param use_standby_node: If True, send the request to a standby Vault node address
        :type use_standby_node: bool
        :param expected_status_code: The status code code expected in the response from Vault.
        :type expected_status_code: int
        :param seal_first: If True, seal the Vault node(s) before running the test cases.
        :type seal_first: bool
        :param ha_required: If True, skip the test case when consul / a HA Vault integration cluster is unavailable.
        :type ha_required: bool
        """
        if ha_required and not self.enable_vault_ha:
            # Conditional to allow folks to run this test class without requiring consul to be installed locally.
            self.skipTest("Skipping test case, Vault HA required but not available.")
        if seal_first:
            # Standby nodes can't be sealed directly.
            # I.e.: "vault cannot seal when in standby mode; please restart instead"
            self.manager.restart_vault_cluster()

        # Grab a Vault node address for our desired standby status and create a one-off client configured for that address.
        vault_addr = self.get_vault_addr_by_standby_status(
            standby_status=use_standby_node
        )
        logging.debug("vault_addr being used: %s" % vault_addr)
        client = create_client(url=vault_addr)

        read_status_response = client.sys.read_health_status(
            method=method,
        )
        logging.debug("read_status_response: %s" % read_status_response)
        if expected_status_code == 200:
            self.assertTrue(read_status_response)
        else:
            self.assertEqual(
                first=read_status_response.status_code,
                second=expected_status_code,
            )
        if method != "HEAD":
            if not isinstance(read_status_response, dict):
                read_status_response = read_status_response.json()
            self.assertTrue(expr=read_status_response["initialized"])



#!/usr/bin/env python
"""Token methods module."""
from hvac import utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "token"


class Token(VaultApiBase):
    """Token Auth Method (API).

    Reference: http://localhost:3000/api-docs/auth/token
    """

    def create(
        self,
        id=None,
        role_name=None,
        policies=None,
        meta=None,
        no_parent=False,
        no_default_policy=False,
        renewable=True,
        ttl=None,
        type=None,
        explicit_max_ttl=None,
        display_name="token",
        num_uses=0,
        period=None,
        entity_alias=None,
        wrap_ttl=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create a new token.

        Certain options are only available when called by a root token. If used
        via the /auth/token/create-orphan endpoint, a root token is not required
        to create an orphan token (otherwise set with the no_parent option). If
        used with a role name in the path, the token will be created against the
        specified role name; this may override options set during this call.


        :param id: The ID of the client token. Can only be specified by a root token.
            The ID provided may not contain a `.` character. Otherwise, the
            token ID is a randomly generated value.
        :type id: str
        :param role_name: The name of the token role.
        :type role_name: str
        :param policies: A list of policies for the token. This must be a
            subset of the policies belonging to the token making the request, unless root.
            If not specified, defaults to all the policies of the calling token.
        :type policies: list
        :param meta: A map of string to string valued metadata. This is
            passed through to the audit devices.
        :type meta: map
        :param no_parent: This argument only has effect if used by a root or sudo caller.
            When set to `True`, the token created will not have a parent.
        :type no_parent: bool
        :param no_default_policy: If `True` the default policy will not be contained in this token's policy set.
        :type no_default_policy: bool
        :param renewable:  Set to false to disable the ability of the token to be renewed past its initial TTL.
            Setting the value to true will allow the token to be renewable up to the system/mount maximum TTL.
        :type renewable: bool
        :param ttl: The TTL period of the token, provided as "1h", where hour is the largest suffix. If not provided,
            the token is valid for the default lease TTL, or indefinitely if the root policy is used.
        :type ttl: str
        :param type: The token type. Can be "batch" or "service". Defaults to the type
            specified by the role configuration named by role_name.
        :type type: str
        :param explicit_max_ttl: If set, the token will have an explicit max TTL set upon it.
            This maximum token TTL cannot be changed later, and unlike with normal tokens, updates to the system/mount
            max TTL value will have no effect at renewal time -- the token will never be able to be renewed or used past
            the value set at issue time.
        :type explicit_max_ttl: str
        :param display_name: The display name of the token.
        :type display_name: str
        :param num_uses: The maximum uses for the given token. This can be
            used to create a one-time-token or limited use token. The value of 0 has no
            limit to the number of uses.
        :type num_uses: int
        :param period: If specified, the token will be periodic; it will have
            no maximum TTL (unless an "explicit-max-ttl" is also set) but every renewal
            will use the given period. Requires a root token or one with the sudo capability.
        :type period: str
        :param entity_alias: Name of the entity alias to associate with during token creation.
            Only works in combination with role_name argument and used entity alias must be listed in
            `allowed_entity_aliases`. If this has been specified, the entity will not be inherited from the parent.
        :type entity_alias: str
        :param wrap_ttl: Specifies response wrapping token creation with duration. IE: '15s', '20m', '25h'.
        :type wrap_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "id": id,
                "policies": policies,
                "meta": meta,
                "no_parent": no_parent,
                "no_default_policy": no_default_policy,
                "renewable": renewable,
                "ttl": ttl,
                "type": type,
                "explicit_max_ttl": explicit_max_ttl,
                "display_name": display_name,
                "num_uses": num_uses,
                "period": period,
                "entity_alias": entity_alias,
            }
        )

        api_path = f"/v1/auth/{mount_point}/create"

        if role_name is not None:
            api_path = f"{api_path}/{role_name}"

        return self._adapter.post(
            url=api_path,
            json=params,
            wrap_ttl=wrap_ttl,
        )

    def create_orphan(
        self,
        id=None,
        role_name=None,
        policies=None,
        meta=None,
        no_default_policy=False,
        renewable=True,
        ttl=None,
        type=None,
        explicit_max_ttl=None,
        display_name="token",
        num_uses=0,
        period=None,
        entity_alias=None,
        wrap_ttl=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create a new orphaned token.

        Creates a token via the /auth/token/create-orphan endpoint. A root token
        is not required to create an orphan token with this endpoint (otherwise
        an orphaned token can be set with the `create` method's `no_parent` option).


        :param id: The ID of the client token. Can only be specified by a root token.
            The ID provided may not contain a `.` character. Otherwise, the
            token ID is a randomly generated value.
        :type id: str
        :param role_name: The name of the token role.
        :type role_name: str
        :param policies: A list of policies for the token. This must be a
            subset of the policies belonging to the token making the request, unless root.
            If not specified, defaults to all the policies of the calling token.
        :type policies: list
        :param meta: A map of string to string valued metadata. This is
            passed through to the audit devices.
        :type meta: map
        :param no_default_policy: If `True` the default policy will not be contained in this token's policy set.
        :type no_default_policy: bool
        :param renewable:  Set to false to disable the ability of the token to be renewed past its initial TTL.
            Setting the value to true will allow the token to be renewable up to the system/mount maximum TTL.
        :type renewable: bool
        :param ttl: The TTL period of the token, provided as `1h`, where hour is the largest suffix. If not provided,
            the token is valid for the default lease TTL, or indefinitely if the root policy is used.
        :type ttl: str
        :param type: The token type. Can be `batch` or `service`. Defaults to the type
            specified by the role configuration named by role_name.
        :type type: str
        :param explicit_max_ttl: If set, the token will have an explicit max TTL set upon it.
            This maximum token TTL cannot be changed later, and unlike with normal tokens, updates to the system/mount
            max TTL value will have no effect at renewal time -- the token will never be able to be renewed or used past
            the value set at issue time.
        :type explicit_max_ttl: str
        :param display_name: The display name of the token.
        :type display_name: str
        :param num_uses: The maximum uses for the given token. This can be
            used to create a one-time-token or limited use token. The value of `0` has no
            limit to the number of uses.
        :type num_uses: int
        :param period: If specified, the token will be periodic; it will have
            no maximum TTL (unless an `explicit-max-ttl` is also set) but every renewal
            will use the given period. Requires a root token or one with the sudo capability.
        :type period: str
        :param entity_alias: Name of the entity alias to associate with during token creation.
            Only works in combination with role_name argument and used entity alias must be listed in
            `allowed_entity_aliases`. If this has been specified, the entity will not be inherited from the parent.
        :type entity_alias: str
        :param wrap_ttl: Specifies response wrapping token creation with duration. IE: `15s`, `20m`, `25h`.
        :type wrap_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "id": id,
                "role_name": role_name,
                "policies": policies,
                "meta": meta,
                "no_default_policy": no_default_policy,
                "renewable": renewable,
                "ttl": ttl,
                "type": type,
                "explicit_max_ttl": explicit_max_ttl,
                "display_name": display_name,
                "num_uses": num_uses,
                "period": period,
                "entity_alias": entity_alias,
            }
        )

        api_path = f"/v1/auth/{mount_point}/create-orphan"
        return self._adapter.post(
            url=api_path,
            json=params,
            wrap_ttl=wrap_ttl,
        )

    def list_accessors(self, mount_point=DEFAULT_MOUNT_POINT):
        """List token accessors.

        This requires sudo capability, and access to it should be tightly controlled
        as the accessors can be used to revoke very large numbers of tokens and their associated leases at once.

        Supported methods:
            LIST: /auth/{mount_point}/accessors.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the list_accessors request.
        :rtype: requests.Response
        """
        api_path = f"/v1/auth/{mount_point}/accessors"
        return self._adapter.list(
            url=api_path,
        )

    def lookup(self, token, mount_point=DEFAULT_MOUNT_POINT):
        """Retrieve information about the client token.

        Supported methods:
            POST: /auth/{mount_point}/lookup.

        :param token: Token to lookup.
        :type token: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the lookup_a request.
        :rtype: requests.Response
        """
        params = {
            "token": token,
        }
        api_path = f"/v1/auth/{mount_point}/lookup"
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def lookup_self(self, mount_point=DEFAULT_MOUNT_POINT):
        """Retrieve information about the current client token.

        Supported methods:
            GET: /auth/{mount_point}/lookup-self.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the lookup_a_self request.
        :rtype: requests.Response
        """
        api_path = f"/v1/auth/{mount_point}/lookup-self"
        return self._adapter.get(
            url=api_path,
        )

    def lookup_accessor(self, accessor, mount_point=DEFAULT_MOUNT_POINT):
        """Retrieve information about the client token from its accessor.

        Supported methods:
            POST: /auth/{mount_point}/lookup-accessor.

        :param accessor: Token accessor to lookup.
        :type accessor: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the lookup_accessor request.
        :rtype: requests.Response
        """
        params = {
            "accessor": accessor,
        }
        api_path = "/v1/auth/{mount_point}/lookup-accessor".format(
            mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def renew(
        self, token, increment=None, wrap_ttl=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Renew a lease associated with a token.

        This is used to prevent the expiration of a token, and the automatic revocation of it.
        Token renewal is possible only if there is a lease associated with it.

        Supported methods:
            POST: /auth/{mount_point}/renew.

        :param token: Token to renew. This can be part of the URL  or the body.
        :type token: str
        :param increment: An optional requested lease increment can be provided.
            This increment may be ignored.
        :type increment: str
        :param wrap_ttl: Specifies response wrapping token creation with duration. IE: '15s', '20m', '25h'.
        :type wrap_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the renew_a request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "token": token,
                "increment": increment,
            }
        )
        api_path = f"/v1/auth/{mount_point}/renew"
        return self._adapter.post(
            url=api_path,
            json=params,
            wrap_ttl=wrap_ttl,
        )

    def renew_self(
        self, increment=None, wrap_ttl=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Renew a lease associated with the calling token.

        This is used to prevent the expiration of a token, and the automatic revocation of it.
        Token renewal is possible only if there is a lease associated with it.

        Supported methods:
            POST: /auth/{mount_point}/renew-self.

        :param increment: An optional requested lease increment can be
            provided. This increment may be ignored.
        :type increment: str
        :param wrap_ttl: Specifies response wrapping token creation with duration. IE: '15s', '20m', '25h'.
        :type wrap_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the renew_a_self request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "increment": increment,
            }
        )
        api_path = f"/v1/auth/{mount_point}/renew-self"
        return self._adapter.post(
            url=api_path,
            json=params,
            wrap_ttl=wrap_ttl,
        )

    def renew_accessor(
        self, accessor, increment=None, wrap_ttl=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Renew a lease associated with a token using its accessor.

        This is used to prevent the expiration of a token, and the automatic revocation of it.
        Token renewal is possible only if there is a lease associated with it.

        Supported methods:
            POST: /auth/{mount_point}/renew-accessor.

        :param accessor: Accessor associated with the token to
            renew.
        :type accessor: str
        :param increment: An optional requested lease increment can be
            provided. This increment may be ignored.
        :type increment: str
        :param wrap_ttl: Specifies response wrapping token creation with duration. IE: '15s', '20m', '25h'.
        :type wrap_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the renew_a_accessor request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "accessor": accessor,
                "increment": increment,
            }
        )
        api_path = "/v1/auth/{mount_point}/renew-accessor".format(
            mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
            wrap_ttl=wrap_ttl,
        )

    def revoke(self, token, mount_point=DEFAULT_MOUNT_POINT):
        """Revoke a token and all child tokens.

        When the token is revoked, all dynamic secrets generated with it are also revoked.

        Supported methods:
            POST: /auth/{mount_point}/revoke.

        :param token: Token to revoke.
        :type token: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the revoke_a request.
        :rtype: requests.Response
        """
        params = {
            "token": token,
        }
        api_path = f"/v1/auth/{mount_point}/revoke"
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def revoke_self(self, mount_point=DEFAULT_MOUNT_POINT):
        """Revoke the token used to call it and all child tokens.

        When the token is revoked, all dynamic secrets generated with it are also revoked.

        Supported methods:
            POST: /auth/{mount_point}/revoke-self.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the revoke_a_self request.
        :rtype: requests.Response
        """
        api_path = f"/v1/auth/{mount_point}/revoke-self"
        return self._adapter.post(url=api_path)

    def revoke_accessor(self, accessor, mount_point=DEFAULT_MOUNT_POINT):
        """Revoke the token associated with the accessor and all the child tokens.

        This is meant for purposes where there is no access to token ID but there is need to
        revoke a token and its children.

        Supported methods:
            POST: /auth/{mount_point}/revoke-accessor.

        :param accessor: Accessor of the token.
        :type accessor: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the revoke_a_accessor request.
        :rtype: requests.Response
        """
        params = {
            "accessor": accessor,
        }
        api_path = "/v1/auth/{mount_point}/revoke-accessor".format(
            mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def revoke_and_orphan_children(self, token, mount_point=DEFAULT_MOUNT_POINT):
        """Revoke a token but not its child tokens.

        When the token is revoked, all secrets generated with it are also revoked.
        All child tokens are orphaned, but can be revoked sub-sequently using /auth/token/revoke/.
        This is a root-protected endpoint.

        Supported methods:
            POST: /auth/{mount_point}/revoke-orphan.

        :param token: Token to revoke.
        :type token: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the revoke_and_orphan_children request.
        :rtype: requests.Response
        """
        params = {
            "token": token,
        }
        api_path = "/v1/auth/{mount_point}/revoke-orphan".format(
            mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_role(self, role_name, mount_point=DEFAULT_MOUNT_POINT):
        """Read the named role configuration.

        Supported methods:
            GET: /auth/{mount_point}/roles/{role_name}.

        :param role_name: The name of the token role.
        :type role_name: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the read_role request.
        :rtype: requests.Response
        """
        api_path = "/v1/auth/{mount_point}/roles/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_roles(
        self,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """List available token roles.

        Supported methods:
            LIST: /auth/{mount_point}/roles.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the list_roles request.
        :rtype: requests.Response
        """
        api_path = f"/v1/auth/{mount_point}/roles"
        return self._adapter.list(
            url=api_path,
        )

    def create_or_update_role(
        self,
        role_name,
        allowed_policies=None,
        disallowed_policies=None,
        orphan=False,
        renewable=True,
        path_suffix=None,
        allowed_entity_aliases=None,
        mount_point=DEFAULT_MOUNT_POINT,
        token_period=None,
        token_explicit_max_ttl=None,
    ):
        """Create (or replace) the named role.

        Roles enforce specific behavior when creating tokens that allow token functionality that is otherwise not
        available or would require sudo/root privileges to access. Role parameters, when set, override any provided
        options to the create endpoints. The role name is also included in the token path, allowing all tokens created
        against a role to be revoked using the `/sys/leases/revoke-prefix` endpoint.

        Supported methods:
            POST: /auth/{mount_point}/roles/{role_name}.

        :param role_name: The name of the token role.
        :type role_name: str
        :param allowed_policies: will be added to the created
            token automatically.
        :type allowed_policies: list
        :param disallowed_policies: being added automatically to created
            tokens.
        :type disallowed_policies: list
        :param orphan: tokens created against this policy will
            be orphan tokens (they will have no parent). As such, they will not be
            automatically revoked by the revocation of any other token.
        :type orphan: bool
        :param renewable: allow
            the token to be renewable up to the system/mount maximum TTL.
        :type renewable: bool
        :param path_suffix:
        :type path_suffix: str
        :param allowed_entity_aliases: not case sensitive.
        :type allowed_entity_aliases: str
        :param token_period: the token will have no maximum TTL, every renewal will use the given period.
        :type token_period: str
        :param token_explicit_max_ttl: the token cannot be renewed past this TTL value.
        :type token_explicit_max_ttl: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the create_or_update_role request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "allowed_policies": allowed_policies,
                "disallowed_policies": disallowed_policies,
                "orphan": orphan,
                "renewable": renewable,
                "path_suffix": path_suffix,
                "allowed_entity_aliases": allowed_entity_aliases,
                "token_period": token_period,
                "token_explicit_max_ttl": token_explicit_max_ttl,
            }
        )
        api_path = "/v1/auth/{mount_point}/roles/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def delete_role(self, role_name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete the named token role.

        Supported methods:
            DELETE: /auth/{mount_point}/roles/{role_name}.

        :param role_name: The name of the token role.
        :type role_name: str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the delete_role request.
        :rtype: requests.Response
        """
        api_path = "/v1/auth/{mount_point}/roles/{role_name}".format(
            mount_point=mount_point,
            role_name=role_name,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def tidy(self, mount_point=DEFAULT_MOUNT_POINT):
        """Perform some maintenance tasks to clean up invalid entries that may remain in the token store.

        On Enterprise, Tidy will only impact the tokens in the specified namespace, or the root namespace if unspecified.

        Supported methods:
            POST: /auth/{mount_point}/tidy.

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str
        :return: The response of the tidy_s request.
        :rtype: requests.Response
        """
        api_path = f"/v1/auth/{mount_point}/tidy"
        return self._adapter.post(
            url=api_path,
        )
